from typing import List

from strenum import StrEnum


class FileIOMode(StrEnum):
    READ = 'r'
    WRITE = 'w'
    WRITE_BINARY = 'wb'
    APPEND = 'a'


ILLEGAL_DIRECTORY_CHARACTERS = r'[:*?<>"\'|]'
ILLEGAL_FILENAME_CHARACTERS = r'[/]'  # More characters will be added in future on case by case basis

PYTHON_SCRIPT_EXTENSION: str = '.py'

SKIPPABLE_DIRECTORIES: List[str] = [
    'build',
    'bin',
    'lib',
    'Lib',
    'dist',
    'lib64',
    'share',
    'MACOSX',
    'Scripts',
    'include',
    'Include',
    '.egg-info',
    'pyvenv.cfg',
    '__pycache__',
]



from enum import IntEnum

from strenum import StrEnum


class Integer(IntEnum):
    NEG_ONE = -1
    ZERO = 0
    ONE = 1
    TWO = 2
    TEN = 10


class Index(IntEnum):
    ZERO = Integer.ZERO
    ONE = Integer.ONE
    LAST = Integer.NEG_ONE


class String(StrEnum):
    SPACE = ' '
    EMPTY = ''
    NEXT_LINE = '\n'
    TAB = '\t'
    FORWARD_SLASH = '/'
    UNDERSCORE = '_'
    PERIOD = '.'
    CARRIAGE_RETURN = '\r'



import functools
from typing import Callable, Any, Type


def retry_on_exception(func: Callable = None,
                       retries: int = 1,
                       exception_type: Type[Exception] = None) -> Any:

    def decorator_retry(function: Callable) -> Callable:

        @functools.wraps(function)
        def wrapper(*args, **kwargs):

            caught_exception = None

            for _ in range(retries):

                try:

                    return function(*args, **kwargs)

                except Exception as caught_exception:

                    if not __should_retry(exception_type, caught_exception, retries):

                        raise caught_exception

            raise caught_exception  # Retry exhausted, raise the last exception

        return wrapper

    # Check if no arguments are provided to the decorator
    if func is None:

        return decorator_retry

    else:

        return decorator_retry(func)


def __should_retry(exception_type, exception, retries):
    """
    Check if the exception should trigger a retry.
    """

    if exception_type is None or isinstance(exception, exception_type):

        return retries > 1

    return False


import time
from datetime import datetime

from ..constants import String


def timeit(function):
    """
    Decorator for measuring function's execution time.
    """

    def measure_time(*args, **kw):

        caught_exception = None
        result = None

        print('Program Start:', datetime.now().strftime("%m/%d/%Y, %H:%M:%S"))

        start_time = int(time.time())

        try:
            result = function(*args, **kw)
        except Exception as e:
            caught_exception = e

        end_time = int(time.time())
        execution_duration = (end_time - start_time)

        hours = execution_duration // 3600
        minutes = (execution_duration - 3600 * hours) // 60
        seconds = execution_duration - 3600 * hours - 60 * minutes

        print("{}Processing time of {}(): {} hours, {} minutes, {} seconds.".format(
            String.NEXT_LINE,
            function.__qualname__,
            hours,
            minutes,
            seconds
        ))

        print('Program End:', datetime.now().strftime("%m/%d/%Y, %H:%M:%S"))

        if caught_exception is not None:
            raise caught_exception

        return result

    return measure_time



from setuptools import setup, find_packages


VERSION = '1.0.1'
DESCRIPTION = 'Common constants used throughout pinjet projects'
AUTHOR = 'abetrack3 (Abrar Shahriar Abeed)'
AUTHOR_EMAIL = '<abrarshahriar2361@gmail.com>'

setup(
    name='pinjet-common',
    version=VERSION,
    description=DESCRIPTION,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    packages=find_packages(where='src'),
    package_dir={'': 'src'},
    install_requires=[
        'StrEnum'
    ],
)
